part of etcd;

mixin DelegatingList<E> implements ListBase<E> {
  List<E> _listDelegate; // -- extending class should override this property with the list to which method calls will be delegated

  @override bool             any(bool Function(E element) test)                                      => _listDelegate.any(test);
  @override bool             every(bool Function(E element) test)                                    => _listDelegate.every(test);
  @override bool             contains(Object element)                                                => _listDelegate.contains(element);
  @override E                elementAt(int i)                                                        => _listDelegate.elementAt(i);
  @override Iterable<T>      expand<T>(Iterable<T> Function(E element) f)                            => _listDelegate.expand(f);
  @override E                firstWhere(bool Function(E element) test, {E Function() orElse})        => _listDelegate.firstWhere(test, orElse: orElse);
  @override T                fold<T>(T initialValue, T Function(T previousValue, E element) combine) => _listDelegate.fold(initialValue, combine);
  @override Iterable<E>      followedBy(Iterable<E> other)                                           => _listDelegate.followedBy(other);
  @override void             forEach(void Function(E element) action)                                => _listDelegate.forEach(action);
  @override String           join([String separator = ""])                                           => _listDelegate.join(separator);
  @override E                lastWhere(bool Function(E element) test, {E Function() orElse})         => _listDelegate.lastWhere(test, orElse: orElse);
  @override Iterable<T>      map<T>(T Function(E element) f)                                         => _listDelegate.map(f);
  @override E                reduce(E Function(E previousValue, E element) combine)                  => _listDelegate.reduce(combine);
  @override E                singleWhere(bool Function(E element) test, {E Function() orElse})       => _listDelegate.singleWhere(test, orElse: orElse);
  @override Iterable<E>      skip(int count)                                                         => _listDelegate.skip(count);
  @override Iterable<E>      skipWhile(bool Function(E element) test)                                => _listDelegate.skipWhile(test);
  @override Iterable<E>      take(int count)                                                         => _listDelegate.take(count);
  @override Iterable<E>      takeWhile(bool Function(E element) test)                                => _listDelegate.takeWhile(test);
  @override List<E>          toList({bool growable = true})                                          => _listDelegate.toList(growable: growable);
  @override Set<E>           toSet()                                                                 => _listDelegate.toSet();
  @override Iterable<E>      where(bool Function(E element) test)                                    => _listDelegate.where(test);
  @override Iterable<T>      whereType<T>()                                                          => _listDelegate.whereType<T>();
  @override List<E> operator +(List<E> other)                                                        => _listDelegate + other;
  @override Map<int, E>      asMap()                                                                 => _listDelegate.asMap();
  @override Iterable<E>      getRange(int start, int end)                                            => _listDelegate.getRange(start, end);
  @override int              indexWhere(bool Function(E element) test, [int start = 0])              => _listDelegate.indexWhere(test, start);
  @override int              lastIndexWhere(bool Function(E element) test, [int start])              => _listDelegate.lastIndexWhere(test, start);
  @override List<E>          sublist(int start, [int end])                                           => _listDelegate.sublist(start, end);
  @override int              indexOf(Object element, [int start = 0])                                => _listDelegate.indexOf(element, start);
  @override int              lastIndexOf(Object element, [int start = 0])                            => _listDelegate.lastIndexOf(element, start);
  @override void operator    []=(int index, E value)                                                 => _listDelegate[index] = value;
  @override void             add(E value)                                                            => _listDelegate.add(value);
  @override void             addAll(Iterable<E> iterable)                                            => _listDelegate.addAll(iterable);
  @override void             clear()                                                                 => _listDelegate.clear();
  @override void             fillRange(int start, int end, [E fillValue])                            => _listDelegate.fillRange(start, end, fillValue);
  @override void             insert(int index, E element)                                            => _listDelegate.insert(index, element);
  @override void             insertAll(int at, Iterable<E> iterable)                                 => _listDelegate.insertAll(at, iterable);
  @override bool             remove(Object element)                                                  => _listDelegate.remove(element);
  @override E                removeAt(int index)                                                     => _listDelegate.removeAt(index);
  @override E                removeLast()                                                            => _listDelegate.removeLast();
  @override void             removeRange(int start, int end)                                         => _listDelegate.removeRange(start, end);
  @override void             removeWhere(bool Function(E element) test)                              => _listDelegate.removeWhere(test);
  @override void             replaceRange(int start, int end, Iterable<E> iterable)                  => _listDelegate.replaceRange(start, end, iterable);
  @override void             retainWhere(bool Function(E element) test)                              => _listDelegate.retainWhere(test);
  @override void             setAll(int at, Iterable<E> iterable)                                    => _listDelegate.setAll(at, iterable);
  @override void             setRange(int start, int end, Iterable<E> iterable, [int skipCount = 0]) => _listDelegate.setRange(start, end, iterable);
  @override void             shuffle([Random random])                                                => _listDelegate.shuffle();
  @override void             sort([Comparator<E> compare])                                           => _listDelegate.sort(compare);
  @override E operator       [](int index)                                                           => _listDelegate.elementAt(index);
  @override List<R>          cast<R>()                                                               => _listDelegate.cast<R>();
  @override E            get first                                                                   => _listDelegate.first;
  @override void         set first(E value)                                                          => _listDelegate.first = value;
  @override bool         get isEmpty                                                                 => _listDelegate.isEmpty;
  @override bool         get isNotEmpty                                                              => _listDelegate.isNotEmpty;
  @override Iterator<E>  get iterator                                                                => _listDelegate.iterator;
  @override E            get last                                                                    => _listDelegate.last;
  @override void         set last(E value)                                                           => _listDelegate.last = value;
  @override E            get single                                                                  => _listDelegate.single;
  @override Iterable<E>  get reversed                                                                => _listDelegate.reversed;
  @override int          get length                                                                  => _listDelegate.length;
  @override void         set length(int newLength)                                                   => _listDelegate.length = newLength;
}
